#ifndef INCLUDE_HMMVPMEXUTIL
#define INCLUDE_HMMVPMEXUTIL

#include "mex.h"
#include "HmmvpUtil.hpp"

template<typename T> mxArray* MatrixToMex(const Matrix<T>& m)
  throw (OutOfMemoryException)
{
  int nr = m.Size(1), nc = m.Size(2);
  mxArray* ma = mxCreateDoubleMatrix(nr, nc, mxREAL);
  if (!ma) throw OutOfMemoryException("MatrixToMex");
  double* pma = mxGetPr(ma);
  const T* pm = m.GetPtr();
  for (int i = 0, n = nr*nc; i < n; i++) pma[i] = static_cast<double>(pm[i]);
  return ma;
}

template<typename T> void MexToMatrix(const mxArray* ma, Matrix<T>& m)
  throw (OutOfMemoryException)
{
  int nr = mxGetM(ma), nc = mxGetN(ma);
  m.Resize(nr,nc);
  const double* pma = mxGetPr(ma);
  T* pm = m.GetPtr();
  for (int i = 0, n = nr*nc; i < n; i++) pm[i] = static_cast<T>(pma[i]);
}

template<typename T> void MexToVector
(const mxArray* ma, std::vector<T>& v, T a = 0)
{
  int nr = mxGetM(ma), nc = mxGetN(ma), n = nr*nc;
  if (n == 0) return;
  v.reserve(n);
  double* pa = mxGetPr(ma);
  for (int i = 0; i < n; i++)
    v.push_back(static_cast<T>(pa[i]) + a);
}

// Could specialize MexToVector to T=double

template<typename T> mxArray* VectorToMex
(const std::vector<T>& v, T a = 0)
{
  int n = v.size();
  mxArray* ma = mxCreateDoubleMatrix(1, n, mxREAL);
  double* pma = mxGetPr(ma);
  for (int i = 0; i < n; i++) pma[i] = static_cast<double>(v[i] + a);
  return ma;
}

template<typename T> void SetField
(mxArray* s, int idx, const char* fld, const std::vector<T>& v, T idx_inc = 0)
{
  mxArray* ma = VectorToMex(v, idx_inc);
  mxSetField(s, idx, fld, ma);
}

template<typename T> void SetField
(mxArray* s, int idx, const char* fld, const Matrix<T>& v)
{
  mxArray* ma = MatrixToMex(v);
  mxSetField(s, idx, fld, ma);
}

template<typename T> void SetField
(mxArray* s, int idx, const char* fld, T v)
{
  mxArray* ma = mxCreateDoubleScalar((double)v);
  mxSetField(s, idx, fld, ma);
}

// Provide mex-based SVD in case caller is using Matlab and does not want to
// link to LAPACK and BLAS. Set U, s, or V = 0 if that output is not required.
void MexSvd(const Matd& A, Matd* U, Matd* s, Matd* V);

#endif
